/*-----------------------------------------------------------------------------
 * Name:    GLCD_LPC43xx.c
 * Purpose: LPC43xx low level Graphic LCD (240x320 pixels) using
 *          SPI and LCD controller in RGB mode
 * Note(s):
 *-----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2005-2012 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <LPC43xx.h>                    /* LPC43xx Definitions                */
#include "GLCD.h"
#include "Font_6x8_h.h"
#include "Font_16x24_h.h"

#define FRAME_BUFFER_ADDR 0x28000000

/************************** Orientation  configuration ************************/

#define LANDSCAPE   1                   /* 0 for landscape, 1 for portrait    */
#define ROTATE180   0                   /* 1 to rotate the screen for 180 deg */

/*********************** Hardware specific configuration **********************/

/*------------------------- Speed dependant settings -------------------------*/

/* If processor works on high frequency delay has to be increased, it can be 
   increased by factor 2^N by this constant                                   */
#define DELAY_2N    18

/*---------------------- Graphic LCD size definitions ------------------------*/

#if (LANDSCAPE == 1)
#define WIDTH       480                 /* Screen Width  (in pixels)          */
#define HEIGHT      272                 /* Screen Height (in pixels)          */
#else
#define WIDTH       272                 /* Screen Width  (in pixels)          */
#define HEIGHT      480                 /* Screen Height (in pixels)          */
#endif
#define BPP         16                  /* Bits per pixel    16                */
#define BYPP        ((BPP+7)/8)         /* Bytes per pixel                    */

#define PHYS_XSZ    480                 /* Physical screen width              */
#define PHYS_YSZ    272                 /* Physical screen height             */

/*--------------- Graphic LCD interface hardware definitions -----------------*/

/* Pin CS setting to 0 or 1                                                   */
#define LCD_CS(x)   ((x) ? (LPC_GPIO_PORT->SET[7] = PIN_CS) : (LPC_GPIO_PORT->CLR[7] = PIN_CS))

#define BG_COLOR  0                     /* Background color                   */
#define TXT_COLOR 1                     /* Text color                         */

/* Clock Control Unit register bits */
#define CCU_CLK_CFG_RUN   (1 << 0)
#define CCU_CLK_CFG_AUTO  (1 << 1)
#define CCU_CLK_STAT_RUN  (1 << 0)

/* SCU pin configuration definitions */
#define LCD_PIN_SET ((1 << 7) | (1 << 6) | (1 << 5))
#define LCD_NPR_SET ((1 << 5) | (1 << 4))

/*---------------------------- Global variables ------------------------------*/

/******************************************************************************/
static uint16_t fb[PHYS_XSZ*PHYS_YSZ]__attribute__((at(FRAME_BUFFER_ADDR)));
static uint16_t Color[2] = {White, Black};

/************************ Exported functions **********************************/

/*******************************************************************************
* Initialize the Graphic LCD controller                                        *
*   Parameter:                                                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Init (void) {
  uint32_t i;

  LPC_CGU->BASE_LCD_CLK  = (1    << 11) |
                           (0x10 << 24) ; /* IDIVE is clock source            */

  /* Enable GPIO register interface clock */
  LPC_CCU1->CLK_M4_GPIO_CFG  = CCU_CLK_CFG_AUTO | CCU_CLK_CFG_RUN;
  while (!(LPC_CCU1->CLK_M4_GPIO_STAT & CCU_CLK_STAT_RUN));

  /* Enable LCD clock */
  LPC_CCU1->CLK_M4_LCD_CFG   = CCU_CLK_CFG_AUTO | CCU_CLK_CFG_RUN;
  while (!(LPC_CCU1->CLK_M4_LCD_STAT & CCU_CLK_STAT_RUN));

  /* Configure LCD controller pins -------------------------------------------*/
  LPC_SCU->SFSP4_1  = LCD_PIN_SET | 2;  /* P4_1:  LCD_VD0                     */
	LPC_SCU->SFSP4_4  = LCD_PIN_SET | 2;  /* P4_4:  LCD_VD1                     */
	LPC_SCU->SFSP4_3  = LCD_PIN_SET | 2;  /* P4_3:  LCD_VD2                     */
	LPC_SCU->SFSP4_2  = LCD_PIN_SET | 2;  /* P4_2:  LCD_VD3                     */
	LPC_SCU->SFSP8_7  = LCD_PIN_SET | 3;  /* P8_7:  LCD_VD4                     */
	LPC_SCU->SFSP8_6  = LCD_PIN_SET | 3;  /* P8_6:  LCD_VD5                     */
	LPC_SCU->SFSP8_5  = LCD_PIN_SET | 3;  /* P8_5:  LCD_VD6                     */
	LPC_SCU->SFSP8_4  = LCD_PIN_SET | 3;  /* P8_4:  LCD_VD7                     */
	LPC_SCU->SFSP7_5  = LCD_PIN_SET | 3;  /* P7_5:  LCD_VD8                     */
	LPC_SCU->SFSP4_8  = LCD_PIN_SET | 2;  /* P4_8:  LCD_VD9                     */
	LPC_SCU->SFSP4_10 = LCD_PIN_SET | 2;  /* P4_10: LCD_VD10                    */
	LPC_SCU->SFSP4_9  = LCD_PIN_SET | 2;  /* P4_9:  LCD_VD11                    */
	LPC_SCU->SFSP8_3  = LCD_PIN_SET | 3;  /* P8_3:  LCD_VD12                    */
	LPC_SCU->SFSPB_6  = LCD_PIN_SET | 2;  /* PB_6:  LCD_VD13                    */
	LPC_SCU->SFSPB_5  = LCD_PIN_SET | 2;  /* PB_5:  LCD_VD14                    */
	LPC_SCU->SFSPB_4  = LCD_PIN_SET | 2;  /* PB_4:  LCD_VD15                    */
	LPC_SCU->SFSP7_4  = LCD_PIN_SET | 3;  /* P7_4:  LCD_VD16                    */
	LPC_SCU->SFSP7_3  = LCD_PIN_SET | 3;  /* P7_3:  LCD_VD17                    */
	LPC_SCU->SFSP7_2  = LCD_PIN_SET | 3;  /* P7_2:  LCD_VD18                    */
	LPC_SCU->SFSP7_1  = LCD_PIN_SET | 3;  /* P7_1:  LCD_VD19                    */
	LPC_SCU->SFSPB_3  = LCD_PIN_SET | 2;  /* PB_3:  LCD_VD20                    */
	LPC_SCU->SFSPB_2  = LCD_PIN_SET | 2;  /* PB_2:  LCD_VD21                    */
	LPC_SCU->SFSPB_1  = LCD_PIN_SET | 2;  /* PB_1:  LCD_VD22                    */
	LPC_SCU->SFSPB_0  = LCD_PIN_SET | 2;  /* PB_0:  LCD_VD23                    */
	
  LPC_SCU->SFSP4_5  = LCD_PIN_SET | 2;  /* P4_5:  LCD_VSYNC                   */
  LPC_SCU->SFSP4_6  = LCD_PIN_SET | 2;  /* P4_6:  LCD_EN                      */
  LPC_SCU->SFSP4_7  = LCD_PIN_SET | 0;  /* P4_7:  LCD_DOTCLK                  */

  LPC_SCU->SFSP7_0  = LCD_NPR_SET | 0;  /* P7_0:  LCD_BL_EN                   */
  LPC_SCU->SFSP7_6  = LCD_PIN_SET | 3;  /* P7_6:  LCD_HSYNC                   */

  /* Configure LCD controller ------------------------------------------------*/
  LPC_RGU->RESET_CTRL0 = (1U << 16);
  while ((LPC_RGU->RESET_ACTIVE_STATUS0 & (1U << 16)) == 0);

  LPC_LCD->CTRL &= ~(1 << 0);           /* Disable LCD                        */
  LPC_LCD->INTMSK = 0;                  /* Disable all interrupts             */

  LPC_LCD->UPBASE = FRAME_BUFFER_ADDR;

  LPC_LCD->TIMH  = (40   << 24) |       /* Horizontal back porch              */
                   (5    << 16) |       /* Horizontal front porch             */
                   (2    <<  8) |       /* Horizontal sync pulse width        */
                   (29   <<  2) ;       /* Pixels-per-line 272=16*(16+1) 480=16*(29+1) */
  LPC_LCD->TIMV  = (8    << 24) |       /* Vertical back porch                */
                   (8    << 16) |       /* Vertical front porch               */
                   (2    << 10) |       /* Vertical sync pulse width          */
                   (271  <<  0) ;       /* Lines per panel                    */
  LPC_LCD->POL   = (1    << 26) |       /* Bypass pixel clock divider         */
                   (479  << 16) |       /* Clocks per line: num of LCDCLKs    */
                   (0    << 13) |       /* Invert panel clock                 */
                   (1    << 12) |       /* Invert HSYNC                       */
                   (1    << 11) ;       /* Invert VSYNC                       */
  LPC_LCD->LE    = (1    << 16) |       /* LCDLE Enabled: 1, Disabled: 0      */
                   (9    <<  0) ;       /* Line-end delay: LCDCLK clocks - 1  */
  LPC_LCD->CTRL  = (1    << 11) |       /* LCD Power Enable                   */
									 (1    <<  8) | 
									 (1    <<  5) |       /* 0 = STN display, 1: TFT display    */
                   (4    <<  1) ;       /* Bits per pixel: 16bpp              */

  for (i = 0; i < 256; i++) {
    LPC_LCD->PAL[i] = 0;                /* Clear palette                      */
  }
  LPC_LCD->CTRL |= (1 <<  0);           /* LCD enable                         */

  /* Turn on backlight -------------------------------------------------------*/
  LPC_GPIO_PORT->DIR[3] |= (1 << 8);
  LPC_GPIO_PORT->SET[3] =  (1 << 8);
}


/*******************************************************************************
* Draw a pixel in foreground color                                             *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_PutPixel (unsigned int x, unsigned int y) {
 #if (LANDSCAPE == 0)
  fb[y*PHYS_XSZ + x] = Color[TXT_COLOR];
 #else
  fb[x*PHYS_XSZ + y] = Color[TXT_COLOR];
 #endif
}

/*******************************************************************************
* Set foreground color                                                         *
*   Parameter:      color:    foreground color                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_SetTextColor (unsigned short color) {

  Color[TXT_COLOR] = color;
}


/*******************************************************************************
* Set background color                                                         *
*   Parameter:      color:    background color                                 *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_SetBackColor (unsigned short color) {

  Color[BG_COLOR] = color;
}


/*******************************************************************************
* Clear display                                                                *
*   Parameter:      color:    display clearing color                           *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Clear (unsigned short color) {
  unsigned int i;

  for (i = 0; i < (WIDTH*HEIGHT); i++) {
    fb[i] = color;
  }
}


/*******************************************************************************
* Draw a line at specified coordinates (x,y) with specified length (len) in    *
* given direction (dir).                                                       *
*   Parameter:      x:        x position                                       *
*                   y:        y position                                       *
*                   len:      length                                           *
*                   dir:      direction                                        *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_DrawLine (unsigned int x, unsigned int y, unsigned int len, unsigned int dir) {
  uint32_t i;

  if (dir == 0) {                       /* Vertical line                      */
    for (i = 0; i < len; i++) { GLCD_PutPixel (x, y + i); }
  }
  else {                                /* Horizontal line                    */
    for (i = 0; i < len; i++) { GLCD_PutPixel (x + i, y); }
  }
}


/*******************************************************************************
* Draw character on given position                                             *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   cw:       character width in pixel                         *
*                   ch:       character height in pixels                       *
*                   c:        pointer to character bitmap                      *
*   Return:                                                                    *
*******************************************************************************/
void GLCD_DrawChar (unsigned int x, unsigned int y, unsigned int cw, unsigned int ch, unsigned char *c) {
  int i, j, k, pixs;

  k  = (cw + 7)/8;

  if (k == 1) {
	#if (LANDSCAPE == 0)
    for (j = 0; j < ch; j++) {
      pixs = *(unsigned char *)c;
      c += 1;
    
      for (i = 0; i < cw; i++) {
        fb[(y+j)*PHYS_XSZ + (x+i)] = Color[(pixs >> i) & 1];
      }
    }
	#else
		for (j = ch-1; j >= 0; j--) {
      pixs = *(unsigned char *)c;
      c += 1;
    
      for (i = 0; i < cw; i++) {
        fb[(x+i)*PHYS_XSZ + (y+j)] = Color[(pixs >> i) & 1];
      }
    }
	#endif
  }
  else if (k == 2) {
	#if (LANDSCAPE == 0)
    for (j = 0; j < ch; j++) {
      pixs = *(unsigned short *)c;
      c += 2;
      
      for (i = 0; i < cw; i++) {
        fb[(y+j)*PHYS_XSZ + (x+i)] = Color[(pixs >> i) & 1];
      }
    }
	#else
		for (j = ch-1; j >= 0; j--){
      pixs = *(unsigned short *)c;
      c += 2;
      
      for (i = 0; i < cw; i++) {
        fb[(x+i)*PHYS_XSZ + (y+j)] = Color[(pixs >> i) & 1];
      }
    }
	#endif
  }
}

/*******************************************************************************
* Display character on given line                                              *
*   Parameter:      ln:       line number                                      *
*                   col:      column number                                    *
*                   fi:       font index (0 = 6x8, 1 = 16x24)                  *
*                   c:        ascii character                                  *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DisplayChar (unsigned int ln, unsigned int col, unsigned char fi, unsigned char c) {

  c -= 32;
  switch (fi) {
    case 0:  /* Font 6 x 8 */
      GLCD_DrawChar(col *  6, ln *  8,  6,  8, (unsigned char *)&Font_6x8_h  [c * 8]);
      break;
    case 1:  /* Font 16 x 24 */
      GLCD_DrawChar(col * 16, ln * 24, 16, 24, (unsigned char *)&Font_16x24_h[c * 24]);
      break;
  }
}


/*******************************************************************************
* Display string on given line                                                 *
*   Parameter:      ln:       line number                                      *
*                   col:      column number                                    *
*                   fi:       font index (0 = 6x8, 1 = 16x24)                  *
*                   s:        pointer to string                                *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DisplayString (unsigned int ln, unsigned int col, unsigned char fi, unsigned char *s) {

  while (*s) {
    GLCD_DisplayChar(ln, col++, fi, *s++);
  }
}

/*******************************************************************************
* Draw string at given coordinated                                             *
*   Parameter:      x:        x coordinate                                     *
*                   y:        y coordinate                                     *
*                   fi:       font index (0 = 6x8, 1 = 16x24)                  *
*                   s:        pointer to string                                *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_DrawString (unsigned int x, unsigned int y, unsigned char fi, unsigned char *s) {
  unsigned char c;
  unsigned int  cx = x;

  while (*s) {
    c  = *s++;
    c -= 32;
    switch (fi) {
      case 0:  /* Font 6 x 8 */
        GLCD_DrawChar (cx, y,  6,  8, (unsigned char *)&Font_6x8_h  [c * 8]);
        cx += 6;
        break;
      case 1:  /* Font 16 x 24 */
        GLCD_DrawChar (cx, y, 16, 24, (unsigned char *)&Font_16x24_h[c * 24]);
        cx += 16;
        break;
    }
  }
}


/*******************************************************************************
* Clear given line                                                             *
*   Parameter:      ln:       line number                                      *
*                   fi:       font index (0 = 6x8, 1 = 16x24)                  *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_ClearLn (unsigned int ln, unsigned char fi) {
  unsigned char i;
  unsigned char buf[60];

  switch (fi) {
    case 0:  /* Font 6 x 8 */
      for (i = 0; i < (WIDTH+5)/6; i++)
        buf[i] = ' ';
      buf[i+1] = 0;
      break;
    case 1:  /* Font 16 x 24 */
      for (i = 0; i < (WIDTH+15)/16; i++)
        buf[i] = ' ';
      buf[i+1] = 0;
      break;
  }
  GLCD_DisplayString (ln, 0, fi, buf);
}

/*******************************************************************************
* Draw bargraph                                                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        maximum width of bargraph (in pixels)            *
*                   h:        bargraph height                                  *
*                   val:      value of active bargraph (in 1/1024)             *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bargraph (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned int val) {
  int i,j;

  val = (val * w) >> 10;                /* Scale value                        */
  for (i = 0; i < h; i++) {
    for (j = 0; j <= w-1; j++) {
     #if (LANDSCAPE == 0)
      if(j >= val) {
        fb[(y+i)*PHYS_XSZ + (x+j)] = Color[BG_COLOR];
      } else {
        fb[(y+i)*PHYS_XSZ + (x+j)] = Color[TXT_COLOR];
      }
     #else
      if(j >= val) {
        fb[(x+j)*PHYS_XSZ + (y+i)] = Color[BG_COLOR];
      } else {
        fb[(x+j)*PHYS_XSZ + (y+i)] = Color[TXT_COLOR];
      }
     #endif
    }
  }
}


/*******************************************************************************
* Display graphical bitmap image at position x horizontally and y vertically   *
* (This function is optimized for 16 bits per pixel format, it has to be       *
*  adapted for any other bits per pixel format)                                *
*   Parameter:      x:        horizontal position                              *
*                   y:        vertical position                                *
*                   w:        width of bitmap                                  *
*                   h:        height of bitmap                                 *
*                   bitmap:   address at which the bitmap data resides         *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_Bitmap (unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned char *bitmap) {
  int i, j, ofs;
  unsigned short *bitmap_ptr = (unsigned short *)bitmap;

  for (i = (h-1)*w, ofs = 0; i >= 0; i -= w, ofs++) {    
    for (j = 0; j < w; j++) {
     #if (LANDSCAPE == 0)
      fb[(x+ofs)*PHYS_XSZ + (y+j)] = bitmap_ptr[i+j];
     #else
      fb[(x+j)*PHYS_XSZ + (y+ofs)] = bitmap_ptr[i+j];
     #endif
    }
  }
}



/*******************************************************************************
* Scroll content of the whole display for dy pixels vertically                 *
*   Parameter:      dy:       number of pixels for vertical scroll             *
*   Return:                                                                    *
*******************************************************************************/

void GLCD_ScrollVertical (unsigned int dy) {
#if (LANDSCAPE == 0)
  uint32_t x, y;
  for (y = 0; y < (PHYS_YSZ - dy); y++) {
    for (x = 0; x < PHYS_XSZ; x++) {
      fb[y*PHYS_XSZ + x] = fb[(y+dy)*PHYS_XSZ + x];
    }
  }
  for (; y < PHYS_YSZ; y++) {
    for (x = 0; x < PHYS_XSZ; x++) {
      fb[y*PHYS_XSZ + x] = Color[BG_COLOR];
    }
  }
#endif
}

/******************************************************************************/

