/*------------------------------------------------------------------------------
 *      RL-ARM
 *------------------------------------------------------------------------------
 *      Name:    Serial.c
 *      Purpose: Serial Input Output for NXP LPC43xx
 *      Note(s): Possible defines to select the used communication interface:
 *               __DBG_ITM   - ITM SWO interface (USART is used by default)
 *                           - UART3 interface  (default)
 *------------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2004-2012 KEIL - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <LPC43xx.h>
#include "Serial.h"

#ifdef __DBG_ITM
volatile int32_t ITM_RxBuffer;
#endif


/** Port offset definition */
#define PORT_OFFSET 	0x80
/** Pin offset definition */
#define PIN_OFFSET  	0x04

/* Pin mode defines, following partly a definition from older chip architectures */
#define MD_PUP  (0x0 << 3)
#define MD_BUK  (0x1 << 3)
#define MD_PLN  (0x2 << 3)
#define MD_PDN  (0x3 << 3)
#define MD_EHS  (0x1 << 5)
#define MD_EZI  (0x1 << 6)
#define MD_ZI   (0x1 << 7)    
#define MD_EHD0 (0x1 << 8)
#define MD_EHD1 (0x1 << 9)
#define MD_EHD2 (0x3 << 8)
#define MD_PLN_FAST (MD_PLN | MD_EZI | MD_ZI | MD_EHS



#define U0_RXD_Pin_name_PORT       0X0F
#define U0_RXD_Pin_name_BIT        11
#define U0_RXD_pin_Function       1

#define U0_TXD_Pin_name_PORT       0X0F
#define U0_TXD_Pin_name_BIT        10
#define U0_TXD_pin_Function       1

void scu_pinmux(uint8_t port, uint8_t pin, uint8_t mode, uint8_t func)
{
  uint32_t * scu_base=(uint32_t*)(LPC_SCU_BASE);
  scu_base[(PORT_OFFSET*port+PIN_OFFSET*pin)/4]=mode+func;
} /* scu_pinmux */


/*------------------------------------------------------------------------------
 *       SER_Init:  Initialize Serial Interface
 *----------------------------------------------------------------------------*/

void SER_Init (void) {

#ifdef __DBG_ITM
  ITM_RxBuffer = ITM_RXBUFFER_EMPTY;
#else
  /* Enable GPIO register interface clock                                     */
  LPC_CCU1->CLK_M4_GPIO_CFG     |= 1;
  while (!(LPC_CCU1->CLK_M4_GPIO_STAT   & 1));

  /* Enable USART3 peripheral clock                                           */
  LPC_CCU2->CLK_APB0_USART0_CFG |= 1;
  while (!(LPC_CCU2->CLK_APB0_USART0_STAT & 1));

  /* Enable USART3 register interface clock                                   */
  LPC_CCU1->CLK_M4_USART0_CFG   |= 1;
  while (!(LPC_CCU1->CLK_M4_USART0_STAT & 1));

	scu_pinmux(U0_TXD_Pin_name_PORT ,U0_TXD_Pin_name_BIT , MD_PDN, U0_TXD_pin_Function); 	
	scu_pinmux(U0_RXD_Pin_name_PORT ,U0_RXD_Pin_name_BIT , MD_PLN|MD_EZI|MD_ZI, U0_RXD_pin_Function); 

											                            

  /* Init USART3                                                              */
  LPC_USART0->LCR    = 0x83;            /* 8 bits, no Parity, 1 Stop bit      */
  LPC_USART0->DLL    = 0x06;            /* 115200 Baudrate @ 12 MHz IRC       */
  LPC_USART0->DLM    = 0x00;
  LPC_USART0->FDR    = 0xC1;
  LPC_USART0->LCR    = 0x03;            /* DLAB = 0                           */
#endif
}


/*------------------------------------------------------------------------------
 *       SER_PutChar:  Write a character to the Serial Port
 *----------------------------------------------------------------------------*/

int32_t SER_PutChar (int32_t ch) {

#ifdef __DBG_ITM
  int i;
  ITM_SendChar (ch & 0xFF);
  for (i = 10000; i; i--);
#else
  while (!(LPC_USART0->LSR & 0x20));
  LPC_USART0->THR = ch;
#endif
  return (ch);
}


/*------------------------------------------------------------------------------
 *       SER_GetChar:  Read a character from the Serial Port (non-blocking)
 *----------------------------------------------------------------------------*/

int32_t SER_GetChar (void) {

#ifdef __DBG_ITM
  if (ITM_CheckChar())
    return (ITM_ReceiveChar());
#else
  if (LPC_USART0->LSR & 0x01) 
    return (LPC_USART0->RBR);
#endif
  return (-1);
}

/*------------------------------------------------------------------------------
 * End of file
 *----------------------------------------------------------------------------*/
